@@ -60,18 +60,32 @@ cases where the choice list is dynamically generated when the user hits tab.
 
         parser.add_argument('-o', '--options', choices_function=my_choices_function)
 
-``choices_method`` - this is exactly like choices_function, but the function
-needs to be an instance method of a cmd2-based class. When ArgparseCompleter
-calls the method, it will pass the app instance as the self argument. This is
-good in cases where the list of choices being generated relies on state data of
-the cmd2-based app
+``choices_method`` - this is equivalent to choices_function, but the function
+needs to be an instance method of a cmd2.Cmd or cmd2.CommandSet subclass. When
+ArgparseCompleter calls the method, it well detect whether is is bound to a
+CommandSet or Cmd subclass.
+If bound to a cmd2.Cmd subclass, it will pass the app instance as the `self`
+argument. This is good in cases where the list of choices being generated
+relies on state data of the cmd2-based app.
+If bound to a cmd2.CommandSet subclass, it will pass the CommandSet instance
+as the `self` argument, and the app instance as the positional argument.
 
-    Example::
+    Example bound to cmd2.Cmd::
 
         def my_choices_method(self):
             ...
             return my_generated_list
 
+        parser.add_argument("arg", choices_method=my_choices_method)
+
+    Example bound to cmd2.CommandSEt::
+
+        def my_choices_method(self, app: cmd2.Cmd):
+            ...
+            return my_generated_list
+
+        parser.add_argument("arg", choices_method=my_choices_method)
+
 ``completer_function`` - pass a tab completion function that does custom
 completion. Since custom tab completion operations commonly need to modify
 cmd2's instance variables related to tab completion, it will be rare to need a
@@ -84,10 +98,16 @@ completer function. completer_method should be used in those cases.
             return completions
         parser.add_argument('-o', '--options', completer_function=my_completer_function)
 
-``completer_method`` - this is exactly like completer_function, but the
-function needs to be an instance method of a cmd2-based class. When
-ArgparseCompleter calls the method, it will pass the app instance as the self
-argument. cmd2 provides a few completer methods for convenience (e.g.,
+``completer_method`` - this is equivalent to completer_function, but the function
+needs to be an instance method of a cmd2.Cmd or cmd2.CommandSet subclass. When
+ArgparseCompleter calls the method, it well detect whether is is bound to a
+CommandSet or Cmd subclass.
+If bound to a cmd2.Cmd subclass, it will pass the app instance as the `self`
+argument. This is good in cases where the list of choices being generated
+relies on state data of the cmd2-based app.
+If bound to a cmd2.CommandSet subclass, it will pass the CommandSet instance
+as the `self` argument, and the app instance as the positional argument.
+cmd2 provides a few completer methods for convenience (e.g.,
 path_complete, delimiter_complete)
 
     Example::
@@ -192,11 +212,15 @@ and will not include the description value of the CompletionItems.
 completion and enables nargs range parsing. See _add_argument_wrapper for
 more details on these arguments.
 
-``argparse.ArgumentParser._get_nargs_pattern`` - adds support to for nargs
-ranges. See _get_nargs_pattern_wrapper for more details.
+``argparse.ArgumentParser._get_nargs_pattern`` - adds support for nargs ranges.
+See _get_nargs_pattern_wrapper for more details.
 
-``argparse.ArgumentParser._match_argument`` - adds support to for nargs ranges.
+``argparse.ArgumentParser._match_argument`` - adds support for nargs ranges.
 See _match_argument_wrapper for more details.
+
+``argparse._SubParsersAction.remove_parser`` - new function which removes a
+sub-parser from a sub-parsers group. See _SubParsersAction_remove_parser for
+more details.
 """
 
 import argparse
@@ -528,6 +552,42 @@ def _match_argument_wrapper(self, action, arg_strings_pattern) -> int:
 # noinspection PyProtectedMember
 argparse.ArgumentParser._match_argument = _match_argument_wrapper
 
+
+############################################################################################################
+# Patch argparse._SubParsersAction to add remove_parser function
+############################################################################################################
+
+def _SubParsersAction_remove_parser(self, name: str):
+    """
+    Removes a sub-parser from a sub-parsers group
+
+    This is a custom method being added to the argparse._SubParsersAction
+    class so cmd2 can remove subcommands from a parser.
+
+    :param self: instance of the _SubParsersAction being edited
+    :param name: name of the sub-parser to remove
+    """
+    for choice_action in self._choices_actions:
+        if choice_action.dest == name:
+            self._choices_actions.remove(choice_action)
+            break
+
+    subparser = self._name_parser_map[name]
+    to_remove = []
+    for name, parser in self._name_parser_map.items():
+        if parser is subparser:
+            to_remove.append(name)
+    for name in to_remove:
+        del self._name_parser_map[name]
+
+    if name in self.choices:
+        del self.choices[name]
+
+
+# noinspection PyProtectedMember
+setattr(argparse._SubParsersAction, 'remove_parser', _SubParsersAction_remove_parser)
+
+
 ############################################################################################################
 # Unless otherwise noted, everything below this point are copied from Python's
 # argparse implementation with minor tweaks to adjust output.
@@ -728,14 +788,40 @@ class Cmd2HelpFormatter(argparse.RawTextHelpFormatter):
 class Cmd2ArgumentParser(argparse.ArgumentParser):
     """Custom ArgumentParser class that improves error and help output"""
 
-    def __init__(self, *args, **kwargs) -> None:
-        if 'formatter_class' not in kwargs:
-            kwargs['formatter_class'] = Cmd2HelpFormatter
-
-        super().__init__(*args, **kwargs)
+    def __init__(self,
+                 prog=None,
+                 usage=None,
+                 description=None,
+                 epilog=None,
+                 parents=None,
+                 formatter_class=Cmd2HelpFormatter,
+                 prefix_chars='-',
+                 fromfile_prefix_chars=None,
+                 argument_default=None,
+                 conflict_handler='error',
+                 add_help=True,
+                 allow_abbrev=True) -> None:
+        super(Cmd2ArgumentParser, self).__init__(
+            prog=prog,
+            usage=usage,
+            description=description,
+            epilog=epilog,
+            parents=parents if parents else [],
+            formatter_class=formatter_class,
+            prefix_chars=prefix_chars,
+            fromfile_prefix_chars=fromfile_prefix_chars,
+            argument_default=argument_default,
+            conflict_handler=conflict_handler,
+            add_help=add_help,
+            allow_abbrev=allow_abbrev)
 
     def add_subparsers(self, **kwargs):
-        """Custom override. Sets a default title if one was not given."""
+        """
+        Custom override. Sets a default title if one was not given.
+
+        :param kwargs: additional keyword arguments
+        :return: argparse Subparser Action
+        """
         if 'title' not in kwargs:
             kwargs['title'] = 'subcommands'
 
